#define T_FLOAT 1
#define T_INT 2

#define I_Vm 0
#define I_CoK 1
#define I_CiK 2
#define I_CoNa 3
#define I_CiNa 4
#define I_B 5
/* Storage locations for excitatory activation state 
 */
#define I_AMPA 6
#define I_NMDA 7
/* Storage locations for inhibitory activation state
 */
#define I_GA 6
#define I_GB1 7
#define I_GB2 8

#define I_USER 9

#define N_CHVARS (I_USER+4+1)

struct hh_net {
  int n;			/* Number of units */
  fp_t dT;			/* Integration time step */
  fp_t Vth;
  fp_t nsd;
  fp_t nmu;
  fp_t Pmax;
  fp_t Psd;
  fp_t CoMg;			/* extracellular Mg2+ */
  fp_t TX;			/* magic "X" recovery time */
  fp_t dX;			/* delta-X per firing */
  fp_t Tr;			/* inactivation recovery time */
  fp_t dD;			/* delta-D per firing */
  fp_t ENMDA;			/* Reversal potential for NMDA  */
  fp_t EGABAA;			/* Reversal potential for GABA A */
  fp_t EAMPA;			/* Reversal potential for AMPA */
  fp_t gNMDA;			/* Maximal conductance for NMDA */
  fp_t gGABAA;			/* Maximal conductance for GABA A */
  fp_t gAMPA;			/* Maximal conductance for AMPA */
  fp_t gGABAB;			/* Maximal conductance for GABA B */
  fp_t phi;			/* WB96 'phi' parameter */
  fp_t cm;			/* Membrane capacitance, microF/cm^2 */
  fp_t EL;			/* reversal potential of leak current, mV */
  fp_t EK;			/* reversal potential of K+, mV */
  fp_t ENa;			/* reversal potential of Na+, mV */
  fp_t ECa;			/* reversal potential of Ca2+, mV */
  fp_t gK;			/* K+ conductance, mS/cm^2 */
  fp_t gNa;			/* Na+ conductance, mS/cm^2 */
  fp_t gL;			/* Leak conductance, mS/cm^2 */
  fp_t degC;			/* Temperature, degrees Celsius */
  int  ninh;			/* fraction inhibitory */
  fp_t gKdrx;			/* Maximal conductance of the K+
				   delayed rectifier current for
				   excitatory neurons, mS/cm^2*/
  fp_t gKslowx;			/* Maximal conductance of the slow K+
				   current for excitatory neurons, mS/cm^2 */
  fp_t gKAx;			/* Maximal conductance of the A-type K+
				   current for excitatory neurons, mS/cm^2 */
  fp_t gNaPx;			/* Maximal conductance of the
				   persistent sodium current for
				   excitatory neurons, mS/cm^2*/
  fp_t gNax;			/* Maximal conductance of the fast sodium
				   current for excitatory neurons, mS/cm^2 */

  fp_t gLx;			/* Conductance of the leak current for 
				   excitatory neurons, mS/cm^2 */

  /* Potassium concentration constants, after Bazhenov et al. 2004 */

  fp_t Bmax;			/* Maximum concentration in buffer, mM */
  fp_t Imax;			/* Maximum inward K+ pump, uA/cm^2 */
  fp_t Ko_th;			/* K+ threshold for k2 calculation */

  /* Miscellaneous parameters */
  fp_t Xinit;			/* Initial value for X[] */
  fp_t Prate;			/* Poisson firing rate */

  fp_t a_growth;		/* "Growth" rate for AMPA */
  fp_t a_decay;			/* "Decay" rate for AMPA */
  fp_t n_growth;		/* "Growth" rate for NMDA */
  fp_t n_decay;			/* "Decay" rate for NMDA */
  fp_t r_growth;		/* "Growth" rate for GABA_A */
  fp_t r_decay;			/* "Decay" rate for GABA_A */
  fp_t rb_growth;		/* "Growth" rate for GABA_B */
  fp_t rb_decay;		/* "Decay" rate for GABA_B */

  /* State variables */
  fp_t *Im;			/* Membrane current, uA/cm^2 */
  fp_t *chvars;			/* Channel state variables */
  fp_t *Tlast;			/* Time of last spike */
  char *F;			/* true if firing */
  char *active;			/* true if active (not refractory) */
  fp_t *D;			/* inactivation decay */
  fp_t *Isyn;			/* synaptic input */
  char *inh;			/* TRUE if inhibitory unit */
  fp_t *O;			/* synaptic state variables */
  fp_t *OB;			/* synaptic state variables */
  fp_t *X;			/* "very slow" inactivation */
};

#if 0
extern struct iafs_net *init_iafs(const char *);

extern void free_iafs(struct iafs_net *);

extern void step_iafs(struct iafs_net *, struct graph *, fp_t, fp_t *);
#endif
extern struct hh_net *init_hh(const char *);

extern void free_hh(struct hh_net *);

extern void step_hh(struct hh_net *, struct graph *, fp_t, fp_t *);

extern void init_exc_act(struct hh_net *net_p);
extern void init_inh_act(struct hh_net *net_p);
