/*
 * Procedure for simple Runge-Kutta 4th order finite-difference integration
 */
void rk4(int n, 
	 ode_func_t fun,
	 void *argp,
	 double dt, 
	 double x, 
	 double *y, 
	 double *dy, 
	 double *s, 
	 double *yk,
	 double *K)
{
  int i;
  double xk;

  fun(n, argp, x, y, dy, K);

  for (i = 0; i < n; i++) { 
    s[i] = dy[i];
    yk[i] = y[i] + (dt / 2.0) * dy[i];
  }

  xk = x + dt / 2.0;

  fun(n, argp, xk, yk, dy, K);

  for (i = 0; i < n; i++) { 
    s[i] += 2.0 * dy[i]; 
    yk[i] = y[i] + (dt / 2.) * dy[i]; 
  }

  fun(n, argp, xk, yk, dy, K);

  for (i = 0; i < n; i++) { 
    s[i] += 2.0 * dy[i]; 
    yk[i] = y[i] + dt * dy[i];
  }

  xk = x + dt;
  fun(n, argp, xk, yk, dy, K);

  for (i = 0; i < n; i++) { 
    y[i] += (dt / 6.0) * (s[i] + dy[i]);
  }
}

